/***************************************************************************
                          lum-prefs.c  -  Lumiere Preferences Window
                             -------------------
    begin                : jeu jan 16 02:08:47 BST 2002
    copyright            : (C) 2003 by Stphane Konstantaropoulos
    email                : ssk01@aber.ac.uk | stephanek@brutele.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
 * CVS block
 *
 * $Author: stephanek $
 * $Revision: 1.4 $
 * $Date: 2003/02/11 02:05:15 $
 * $Log: lum-prefs.c,v $
 * Revision 1.4  2003/02/11 02:05:15  stephanek
 * 0.4pre
 * ready.
 *
 * preferences adapted to control (backend, xvideo added; audio removed)
 * tested enough.
 * GO
 *
 * Revision 1.3  2003/01/18 22:04:42  stephanek
 * version 0.3 ready
 * use of glade wherever I can
 * GConf stuff ready, more to be added
 *
 * Revision 1.2  2003/01/16 11:40:54  stephanek
 * started preferences. NOT working yet
 * Use of Glade for the GUI
 *
 * Revision 1.1.1.1  2003/01/03 02:09:01  stephanek
 * Imported sources
 *
 */

#include <config.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libgnomeui/libgnomeui.h>

#include "lum-prefs.h"

#define GCONF_LUM_PLAYER_DIR "/apps/lumiere/player"
#define GCONF_LUM_PLAYER_XV "/apps/lumiere/player/xv"
#define GCONF_LUM_PLAYER_SLANG "/apps/lumiere/player/slang"
#define GCONF_LUM_PLAYER_ALANG "/apps/lumiere/player/alang"
#define GCONF_LUM_PLAYER_DVDDEV "/apps/lumiere/player/dvd_dev"
#define GCONF_LUM_PLAYER_DOUBLE "/apps/lumiere/player/double"
#define GCONF_LUM_PLAYER_INDEX "/apps/lumiere/player/index"
#define GCONF_LUM_PLAYER_RTC "/apps/lumiere/player/rtc"
#define GCONF_LUM_PLAYER_SOFTSLEEP "/apps/lumiere/player/softsleep"
#define GCONF_LUM_PLAYER_STOPSSAVER "/apps/lumiere/player/stop_ssaver"
#define GCONF_LUM_PLAYER_DR "/apps/lumiere/player/dr"
#define GCONF_LUM_PLAYER_FFACTOR "/apps/lumiere/player/font_factor"
#define GCONF_LUM_PLAYER_FRAMEDROP "/apps/lumiere/player/framedrop"
#define GCONF_LUM_PLAYER_CACHE "/apps/lumiere/player/cache"
#define GCONF_LUM_PLAYER_AUTOQ "/apps/lumiere/player/autoquality"
#define GCONF_LUM_PLAYER_OSD "/apps/lumiere/player/osd"
#define GCONF_LUM_PLAYER_BE "/apps/lumiere/player/backend"

#define BE_KEY "ao_key"
static void hide(GladeXML *xml, gboolean hide);
/*
 * check toggle
 * Valid with all buttons, cool
 */
static void check_toggle_cb (GtkWidget *widget, gpointer data)
{
	char *key = NULL;

	key = g_object_get_data (G_OBJECT (widget), "GCONF_KEY");
	if (key == NULL) return;

	gconf_client_set_bool (GCONF_CLIENT (data),
			       key,
			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)),
			       NULL);
}

/*
 * spin moved
 * Valid with all spin buttons!
 */
static void spin_changed_cb(GtkWidget *widget, gpointer data)
{
	char *key = NULL;

	key = g_object_get_data (G_OBJECT (widget), "GCONF_KEY");
	if (key == NULL) return;

	gconf_client_set_int (GCONF_CLIENT (data),
			       key,
			       gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (widget)),
			       NULL);
}

/*
 * Text changed
 * Valid with all entries!
 */
static void text_changed_cb(GtkWidget *widget, gpointer data)
{
	char *key = NULL;

	key = g_object_get_data (G_OBJECT (widget), "GCONF_KEY");
	if (key == NULL) return;

	gconf_client_set_string (GCONF_CLIENT (data),
			       key,
			       g_strdup(gtk_entry_get_text(GTK_ENTRY (widget))),
			       NULL);
}

/*
 * menu item activated
 * Valid with all entries!
 */
static void submenu_activate_cb(GtkWidget *widget, gpointer data)
{
	char *key;
	char *string;
	GladeXML *xml;

	key = g_object_get_data (G_OBJECT (widget), "GCONF_KEY");
	xml = g_object_get_data (G_OBJECT (widget), "xml");
	
	if (key == NULL) return;
	string = g_object_get_data (G_OBJECT (widget), BE_KEY);

	gconf_client_set_string (GCONF_CLIENT (data),
			       key,
			       g_strdup(string),
			       NULL);
	if(g_ascii_strcasecmp(string, "mplayer"))
		hide(xml, FALSE);
	else
		hide(xml, TRUE);
			
}

/*
 * ffactor menu item activated
 * Valid with all entries!
 */
static void ffmenu_activate_cb(GtkWidget *widget, gpointer data)
{
	char *key;
	float value;
	float tmp_value;

	key = g_object_get_data (G_OBJECT (widget), "GCONF_KEY");
	if (key == NULL) return;
	tmp_value = GPOINTER_TO_INT(
			g_object_get_data (G_OBJECT (widget), BE_KEY));
	if(tmp_value == 75)
		value = 0.75;
	else
		value = (float) tmp_value;

	gconf_client_set_float (GCONF_CLIENT (data), key, value, NULL);
}


static gboolean key_press_cb (
		GtkWidget *dialog, GdkEventKey *event, gpointer user_data)
{
	if (event->keyval == GDK_Escape) {
		gtk_widget_destroy (dialog);
		return TRUE;
	}
	else {
		return FALSE;
	}
}

void lum_prefs_show(GConfClient *client)
{
	GtkWidget *dlg;
	GladeXML *xml;
	GtkWidget *widget;
	GtkWidget *menu;

	xml = glade_xml_new (DATADIR"/lumiere/glade/lumiere.glade",
				"window", "lumiere");
	g_assert(xml != NULL);

	if(g_ascii_strcasecmp(gconf_client_get_string (
					client, GCONF_LUM_PLAYER_BE, NULL), "mplayer"))
		hide(xml, FALSE);

	dlg = glade_xml_get_widget(xml, "window");
	g_signal_connect(G_OBJECT(dlg), "key-press-event",
			G_CALLBACK(key_press_cb), NULL);
	widget = glade_xml_get_widget (xml, "close_button");
	g_signal_connect_swapped (G_OBJECT (widget), "clicked", 
				  G_CALLBACK (gtk_widget_destroy), dlg);
	widget = glade_xml_get_widget (xml, "notebook");
	gtk_notebook_set_current_page(GTK_NOTEBOOK(widget), 3);
	/* the toggle buttons */
	widget = glade_xml_get_widget(xml, "dr_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_DR, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_DR);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "double_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_DOUBLE, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_DOUBLE);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "idx_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_INDEX, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_INDEX);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "rtc_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_RTC, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_RTC);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "soft_sleep_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_SOFTSLEEP, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_SOFTSLEEP);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "ssaver_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_STOPSSAVER, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_STOPSSAVER);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "xv_button");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
		gconf_client_get_bool (client, GCONF_LUM_PLAYER_XV, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_XV);
	g_signal_connect (G_OBJECT (widget), 
			  "toggled", 
			  G_CALLBACK (check_toggle_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "osd_spinbutton");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
			(double) gconf_client_get_int(
					client, GCONF_LUM_PLAYER_OSD, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_OSD);
	g_signal_connect (G_OBJECT (widget), 
			  "value_changed", 
			  G_CALLBACK (spin_changed_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "osd_spinbutton");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
			(double) gconf_client_get_int(
					client, GCONF_LUM_PLAYER_OSD, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_OSD);
	g_signal_connect (G_OBJECT (widget), 
			  "value_changed", 
			  G_CALLBACK (spin_changed_cb), 
			  client);

	widget = glade_xml_get_widget(xml, "autoq_button");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
			(double) gconf_client_get_int(
					client, GCONF_LUM_PLAYER_AUTOQ, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_AUTOQ);
	g_signal_connect (G_OBJECT (widget), 
			  "value_changed", 
			  G_CALLBACK (spin_changed_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "cache_button");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
			(double) gconf_client_get_int(
					client, GCONF_LUM_PLAYER_CACHE, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_CACHE);
	g_signal_connect (G_OBJECT (widget), 
			  "value_changed", 
			  G_CALLBACK (spin_changed_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "drop_button");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
			(double) gconf_client_get_int(
					client, GCONF_LUM_PLAYER_FRAMEDROP, NULL));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_FRAMEDROP);
	g_signal_connect (G_OBJECT (widget), 
			  "value_changed", 
			  G_CALLBACK (spin_changed_cb), 
			  client);
	
	widget = glade_xml_get_widget(xml, "dvd_dev_entry");
	gtk_entry_set_text(GTK_ENTRY(widget), g_strdup(
				gconf_client_get_string(
					client, GCONF_LUM_PLAYER_DVDDEV, NULL)));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_DVDDEV);
	g_signal_connect (G_OBJECT (widget), 
			  "changed", 
			  G_CALLBACK (text_changed_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "alang_entry");
	gtk_entry_set_text(GTK_ENTRY(widget), g_strdup(
				gconf_client_get_string(
					client, GCONF_LUM_PLAYER_ALANG, NULL)));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_ALANG);
	g_signal_connect (G_OBJECT (widget), 
			  "changed", 
			  G_CALLBACK (text_changed_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "slang_entry");
	gtk_entry_set_text(GTK_ENTRY(widget), g_strdup(
				gconf_client_get_string(
					client, GCONF_LUM_PLAYER_SLANG, NULL)));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_SLANG);
	g_signal_connect (G_OBJECT (widget), 
			  "changed", 
			  G_CALLBACK (text_changed_cb), 
			  client);

	menu = glade_xml_get_widget(xml, "be_optionmenu");
	widget = glade_xml_get_widget(xml, "mplayer");
	if(g_strcasecmp("mplayer", gconf_client_get_string(
					client, GCONF_LUM_PLAYER_BE, NULL)) == 0)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 0);

	g_object_set_data (G_OBJECT (widget),
			BE_KEY, "mplayer");
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_BE);
	g_object_set_data (G_OBJECT (widget),"xml", xml);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (submenu_activate_cb), 
		  client);
	widget = glade_xml_get_widget(xml, "xine");
	if(g_strcasecmp("xine", gconf_client_get_string(
					client, GCONF_LUM_PLAYER_BE, NULL)) == 0)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 1);
	g_object_set_data (G_OBJECT (widget),
			BE_KEY, "xine");
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_BE);
	g_object_set_data (G_OBJECT (widget),"xml", xml);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (submenu_activate_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "glplay");
	if(g_strcasecmp("glplay", gconf_client_get_string(
					client, GCONF_LUM_PLAYER_BE, NULL)) == 0)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 2);
	g_object_set_data (G_OBJECT (widget),
			BE_KEY, "glplay");
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_BE);
	g_object_set_data (G_OBJECT (widget),"xml", xml);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (submenu_activate_cb), 
			  client);

	menu = glade_xml_get_widget(xml, "ffactor_menu");
	widget = glade_xml_get_widget(xml, "0");
	if(gconf_client_get_float(client, GCONF_LUM_PLAYER_FFACTOR, NULL) == 0.0f)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 0);

	g_object_set_data (G_OBJECT (widget),
			BE_KEY, "0");
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_FFACTOR);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (ffmenu_activate_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "075");
	if(gconf_client_get_float(client, GCONF_LUM_PLAYER_FFACTOR, NULL) == 0.75f)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 1);

	g_object_set_data (G_OBJECT (widget),
			BE_KEY, GINT_TO_POINTER(75));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_FFACTOR);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (ffmenu_activate_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "1");
	if(gconf_client_get_float(client, GCONF_LUM_PLAYER_FFACTOR, NULL) == 1.0f)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 2);

	g_object_set_data (G_OBJECT (widget),
			BE_KEY, GINT_TO_POINTER(1));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_FFACTOR);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (ffmenu_activate_cb), 
			  client);
	widget = glade_xml_get_widget(xml, "10");
	if(gconf_client_get_float(client, GCONF_LUM_PLAYER_FFACTOR, NULL) == 10.0f)
		gtk_option_menu_set_history(GTK_OPTION_MENU(menu), 3);

	g_object_set_data (G_OBJECT (widget),
			BE_KEY, GINT_TO_POINTER(10));
	g_object_set_data (G_OBJECT (widget),
			"GCONF_KEY", GCONF_LUM_PLAYER_FFACTOR);
	g_signal_connect (G_OBJECT (widget), 
			  "activate", 
			  G_CALLBACK (ffmenu_activate_cb), 
			  client);
}

static void hide(GladeXML *xml, gboolean hide){
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "dr_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "double_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "osd_spinbutton"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "autoq_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "dvd_dev_entry"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "alang_entry"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "slang_entry"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "idx_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "rtc_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "soft_sleep_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "ssaver_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "drop_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "cache_button"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "ffactor_menu"), hide);
	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "10"), hide);
	
}
